@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

groupbox
{
  margin : 5px ;
  border : 2px solid ;
  -moz-border-top-colors : #858B97 #ECEFF2 ;
  -moz-border-right-colors : #ECEFF2 #858B97 ;
  -moz-border-bottom-colors : #ECEFF2 #858B97 ;
  -moz-border-left-colors : #858B97 #ECEFF2 ;
  -moz-border-radius : 5px ;
  padding : 8px ;
}

.groupbox-body
{
  padding : inherit ;
}

caption
{
  margin : 0px 6px 2px 6px ;
  padding : 0px 3px ;
  background-color : #C7D0D9 ;
}
